﻿CREATE TABLE [dbo].[Buch] (
    [ID]               INT           IDENTITY (1, 1) NOT NULL,
    [ISBN10]           CHAR (10)     NULL,
    [ISBN13]           CHAR (14)     NULL,
    [Autor]            VARCHAR (80)  NULL,
    [Titel]            VARCHAR (80)  NULL,
    [Verlag]           VARCHAR (80)  NULL,
    [Seiten]           INT           NULL,
    [Auflage]          TINYINT       NULL,
    [Sprache]          VARCHAR (20)  CONSTRAINT [DF_Buch_Sprache] DEFAULT ('Deutsch') NULL,
    [Hardcover]        BIT           NULL,
    [Erscheinungsjahr] INT           NULL,
    [Bewertung]        TINYINT       NULL,
    [Beschreibung]     VARCHAR (MAX) NULL,
    [Kategorie]        VARCHAR (80)  NULL,
    [MediaId]          INT           DEFAULT (NEXT VALUE FOR [dbo].[MediaId]) NULL,
    CONSTRAINT [PK_Buch] PRIMARY KEY CLUSTERED ([ID] ASC)
);


GO
CREATE NONCLUSTERED INDEX [IX_Titel]
    ON [dbo].[Buch]([Titel] ASC);


GO
CREATE NONCLUSTERED INDEX [IX_ISBN10]
    ON [dbo].[Buch]([ISBN10] ASC);


GO
CREATE NONCLUSTERED INDEX [IX_ISBN13]
    ON [dbo].[Buch]([ISBN13] ASC);


GO
CREATE NONCLUSTERED INDEX [IX_Autor]
    ON [dbo].[Buch]([Autor] ASC);


GO
CREATE TRIGGER dbo.TRG_ChangeLog
   ON  dbo.Buch
   AFTER UPDATE, INSERT, DELETE
AS 
BEGIN
	SET NOCOUNT ON;
	
	DECLARE @Aenderungen AS varchar(max)
	DECLARE @IDsInserted AS varchar(max) = ''
	DECLARE @IDsDeleted AS varchar(max) = ''

	SET @IDsInserted=(SELECT STRING_AGG(CONVERT(VARCHAR(20),ID), ', ') FROM INSERTED)
	SET @IDsDeleted=(SELECT STRING_AGG(CONVERT(VARCHAR(20),ID), ', ') FROM DELETED)

	SET @Aenderungen=CONCAT('Hinzugefügte IDs: ', @IDsInserted, ' Gelöschte IDs: ', @IDsDeleted)

	INSERT INTO AenderungsLog (Aenderung)
	VALUES (@Aenderungen)
END

