-- Kapitel 6 --

USE MediaBase
GO

-- Abfragen auf einer Tabelle

SELECT Autor, Titel FROM Buch

SELECT DISTINCT Autor FROM Buch

SELECT * FROM Buch ORDER BY Autor, Titel

SELECT * FROM Buch ORDER BY Autor ASC, Erscheinungsjahr DESC

SELECT * FROM CD WHERE Interpret = 'Queen'

SELECT * FROM CD WHERE Interpret = 'Queen' AND Erscheinungsjahr > 1990
SELECT * FROM CD WHERE Interpret = 'Queen' OR Interpret = 'The Police'

SELECT * FROM CD WHERE Interpret IN ('Queen', 'The Police')

SELECT TOP 10 PERCENT *
FROM CD_Track

SELECT TOP 3 *
FROM CD
ORDER BY Erscheinungsjahr DESC

SET ROWCOUNT 5
SELECT * FROM CD_Track

SET ROWCOUNT 0
SELECT * FROM CD_Track

-- Aggregierungsfunktionen und Gruppierungen

SELECT COUNT(*) FROM Buch WHERE Auflage IS NOT NULL
SELECT COUNT(Auflage) FROM Buch

SELECT COUNT(*) AS AnzahlBuecher 
FROM Buch

SELECT AnzahlBuecher = COUNT(*)
FROM Buch

SELECT COUNT(*) AS Anzahl, Autor 
FROM Buch 
GROUP BY Autor

SELECT COUNT(*) AS Anzahl, Autor 
FROM Buch 
GROUP BY Autor
HAVING COUNT(*)>1

-- Abfragen auf mehreren Tabellen

SELECT *
FROM CD INNER JOIN CD_Track ON CD.ID = CD_Track.idCD

SELECT CD.*, CD_Track.TrackNr, CD_Track.Titel
FROM CD INNER JOIN CD_Track ON CD.ID = CD_Track.idCD

SELECT CD.*, CDT.TrackNr, CDT.Titel
FROM CD INNER JOIN CD_Track AS CDT ON CD.ID = CDT.idCD

SELECT CD.*, CDT.TrackNr, CDT.Titel
FROM CD LEFT JOIN CD_Track AS CDT ON CD.ID = CDT.idCD

SELECT Titel, Interpret, Erscheinungsjahr FROM CD
UNION SELECT Titel, Autor, Erscheinungsjahr FROM Buch
UNION SELECT Titel, Regie, Erscheinungsjahr FROM DVD

SELECT Titel, Interpret AS InterpretAutorRegie, Erscheinungsjahr, 'CD' as Quelle FROM CD
UNION SELECT Titel, Autor, Erscheinungsjahr, 'Buch' FROM Buch
UNION SELECT Titel, Regie, Erscheinungsjahr, 'DVD' FROM DVD

SELECT Titel, Interpret AS InterpretAutorRegie, Erscheinungsjahr, 'CD' as Quelle FROM CD
UNION ALL SELECT Titel, Autor, Erscheinungsjahr, 'Buch' FROM Buch
UNION ALL SELECT Titel, Regie, Erscheinungsjahr, 'DVD' FROM DVD

-- INSERT INTO

INSERT INTO Buch (Autor, Titel, Hardcover, Kategorie)
VALUES ('Johanna Rohling', 'Harald Tpfer und die schlechte bersetzung', 0, 'Roman')

INSERT INTO CD (Titel, Kategorie)
SELECT Titel, 'Soundtrack' FROM DVD

-- SELECT INTO

SELECT * 
INTO MeineBuecher
FROM Buch
WHERE Autor = 'Robert Panther'

SELECT CD.*, CDT.TrackNr, CDT.Titel as TrackTitel
INTO CDMitTracks
FROM CD LEFT JOIN CD_Track as CDT ON CD.ID = CDT.idCD

-- UPDATE

UPDATE Buch
SET Verlag = '(unbekannt)'
WHERE Verlag IS NULL

UPDATE MeineBuecher
SET Seiten = Seiten + 4

UPDATE MeineBuecher
SET Bewertung=1, Beschreibung='Einer der groen Klassiker der Weltliteratur'
WHERE Titel='SQL Server Performance-Ratgeber'

-- DELETE

DELETE FROM MeineBuecher
WHERE Seiten < 300

-- TRUNCATE TABLE

TRUNCATE TABLE MeineBuecher

-- Sichten

CREATE VIEW dbo.VW_CDsAb1990 AS
SELECT ID, Titel, Erscheinungsjahr
FROM dbo.CD
WHERE (Erscheinungsjahr >= 1990)

DROP VIEW dbo.VW_CDsAb1990

GO

CREATE VIEW dbo.VW_CDsAb1990 AS
SELECT ID, Titel, Interpret, Erscheinungsjahr
FROM dbo.CD
WHERE (Erscheinungsjahr >= 1990)

ALTER VIEW dbo.VW_CDsAb1990 AS
SELECT ID,Titel, Interpret, Erscheinungsjahr
FROM dbo.CD
WHERE (Erscheinungsjahr >= 1990)

DROP VIEW IF EXISTS dbo.VW_CDsAb1990

GO

CREATE VIEW dbo.VW_CDsAb1990 AS
SELECT ID, Titel, Interpret, Erscheinungsjahr
FROM dbo.CD
WHERE (Erscheinungsjahr >= 1990)

-- ab SQL 2016 SP1
CREATE OR ALTER VIEW dbo.VW_CDsAb1990 AS
SELECT ID, Titel, Interpret, Erscheinungsjahr
FROM dbo.CD
WHERE (Erscheinungsjahr >= 1990)

--

SELECT * FROM VW_CDsAb1990

SELECT ID, Titel, Interpret, Erscheinungsjahr
FROM dbo.CD
WHERE (Erscheinungsjahr >= 1990)

SELECT * 
FROM VW_CDsAb1990
WHERE (Erscheinungsjahr <= 1995)

SELECT * 
FROM VW_CDsAb1990 AS CD1990
INNER JOIN dbo.CD_Track AS TRK ON CD1990.ID=TRK.idCD

SELECT CD1990.*, TRK.Titel as TrackTitel, TRK.Interpret as TrackInterpret
FROM VW_CDsAb1990 AS CD1990
INNER JOIN dbo.CD_Track AS TRK ON CD1990.ID=TRK.idCD

--

INSERT INTO VW_CDsAb1990 (Interpret, Titel, Erscheinungsjahr)
VALUES ('Erwin Mustermann', 'Best Of Erwin Mustermann', 1970)

ALTER VIEW dbo.VW_CDsAb1990 AS
SELECT ID,Titel, Interpret, Erscheinungsjahr
FROM dbo.CD
WHERE (Erscheinungsjahr >= 1990)
WITH CHECK OPTION

--

ALTER VIEW dbo.VW_AktuelleCDs
AS
SELECT TOP (100) PERCENT dbo.CD.ID, dbo.CD.Titel, dbo.CD.Interpret, dbo.CD.AnzahlCDs, dbo.CD.Erscheinungsjahr, dbo.CD.Bewertung, dbo.CD_Track.CDNr, 
	dbo.CD_Track.TrackNr, dbo.CD_Track.Titel AS TrackTitel, dbo.CD_Track.Interpret AS TrackInterpret, dbo.CD_Track.Dauer, 
	dbo.CD_Track.Bewertung AS TrackBewertung, dbo.CD.Kategorie
FROM dbo.CD
	INNER JOIN dbo.CD_Track ON dbo.CD.ID = dbo.CD_Track.idCD
WHERE dbo.CD.Erscheinungsjahr > 1995

SELECT * FROM VW_AktuelleCDs

DELETE FROM VW_AktuelleCDs WHERE ID=2

INSERT INTO VW_AktuelleCDs (Interpret, Titel, Erscheinungsjahr, TrackNr)
VALUES ('Erwin Mustermann', 'The Very Best Of Erwin Mustermann', 2007, 1)
-- Fehler!

INSERT INTO VW_AktuelleCDs (Interpret, Titel, Erscheinungsjahr)
VALUES ('Erwin Mustermann', 'The Very Best Of Erwin Mustermann', 2007)

INSERT INTO VW_AktuelleCDs (TrackTitel)
VALUES ('SingSang')
-- Fehler!

UPDATE VW_AktuelleCDs
SET Bewertung=1 WHERE ID=1
-- setzt die Bewertung fr die CD

UPDATE VW_AktuelleCDs
SET TrackBewertung=1 where ID=1
-- setzt die Bewertung fr die CD-Tracks

UPDATE VW_AktuelleCDs
SET Bewertung=1, TrackBewertung=1 where ID=1
-- Fehler!

UPDATE VW_AktuelleCDs
SET TrackBewertung=Bewertung
WHERE TrackBewertung IS NULL
