-- Kapitel 7 --

USE MediaBase
GO

-- 7.1 Komplexe SQL-SELECTs --

-- Fallunterscheidung mit CASE

SELECT *, 'Hardcover'
FROM Buch
WHERE Hardcover=1
UNION
SELECT *, 'Softcover'
FROM Buch
WHERE Hardcover=0

SELECT *, CASE
	WHEN Hardcover=1 
	THEN 'Hardcover' 
	ELSE 'Softcover' 
	END AS Cover
FROM Buch

SELECT *, CASE
	WHEN Hardcover=1 THEN 'Hardcover'
	WHEN Hardcover=0 THEN 'Softcover'
	ELSE '(unbekannt)'
	END AS Cover
FROM dbo.Buch

SELECT *, CASE Hardcover
	WHEN 1 THEN 'Hardcover'
	WHEN 0 THEN 'Softcover'
	ELSE '(unbekannt)'
	END AS Cover
FROM dbo.Buch

-- Fallunterscheidung mit IIF und CHOOSE

SELECT *, IIF(Hardcover=1, 'Hardcover', 'Softcover') AS Cover
FROM dbo.Buch

SELECT CHOOSE(2, 'eins', 'zwei', 'drei')

SELECT *, CHOOSE(Laendercode, 'USA', 'Europa/Sdafrika/Naher Osten/Japan', 'Sdost-Asien', 
	'Australien/Mittelamerika/Sdamerika', 'Osteuropa/ehem.UdSSR/Indien/Afrika','China') AS LaendercodeText
FROM dbo.DVD

SELECT *, CHOOSE(Laendercode+1, 'alle Regionen', 'USA', 'Europa/Sdafrika/Naher Osten/Japan', 'Sdost-Asien', 
	'Australien/Mittelamerika/Sdamerika','Osteuropa/ehem.UdSSR/Indien/Afrika', 'China') AS LaendercodeText
FROM dbo.DVD

-- Unterabfragen

SELECT idCD
FROM dbo.CD_Track
GROUP BY idCD
HAVING COUNT(*)>10

SELECT Titel, Interpret
FROM dbo.CD
WHERE ID IN
	(SELECT idCD
	FROM dbo.CD_Track
	GROUP BY idCD
	HAVING COUNT(*)>10)

SELECT Titel, Interpret
FROM dbo.CD
WHERE EXISTS
	(SELECT idCD
	FROM dbo.CD_Track
	WHERE idCD = dbo.CD.ID
	GROUP BY idCD
	HAVING COUNT(*)>10)

SELECT Titel, Interpret
FROM dbo.CD
WHERE EXISTS
	(SELECT idCD
	FROM dbo.CD_Track
	GROUP BY idCD
	HAVING COUNT(*)>10 AND idCD = dbo.CD.ID)

SELECT Titel, Interpret
FROM dbo.CD
WHERE (SELECT COUNT(*)
	FROM dbo.CD_Track
	WHERE idCD = dbo.CD.ID)>10

--

SELECT Titel, Interpret, 
	(SELECT COUNT(*)
	FROM dbo.CD_Track
	WHERE idCD = dbo.CD.ID) AS Anzahl
FROM dbo.CD

SELECT CD.Titel, CD.Interpret, COUNT(CD_Track.ID) as Anzahl
FROM dbo.CD 
	LEFT JOIN dbo.CD_Track ON CD.ID=CD_Track.idCD
GROUP BY CD.Titel, CD.Interpret

SELECT CD.Titel, CD.Interpret, TrackAnzahl.Anzahl
FROM dbo.CD
	LEFT JOIN
		(SELECT idCD, COUNT(*) AS Anzahl
		FROM dbo.CD_Track
		GROUP BY idCD) AS TrackAnzahl
	ON CD.ID = TrackAnzahl.idCD

-- GROUP BY ... ROLLUP

SELECT Interpret, COUNT(*) as Anzahl
FROM dbo.CD 
GROUP BY Interpret

SELECT 'Summe', COUNT(*) as Anzahl
FROM dbo.CD 

SELECT Interpret, COUNT(*) as Anzahl
FROM dbo.CD 
GROUP BY Interpret
UNION
SELECT 'Summe', COUNT(*) as Anzahl
FROM dbo.CD 

-- alte Syntax
SELECT Interpret, COUNT(*) as Anzahl
FROM dbo.CD 
GROUP BY Interpret WITH ROLLUP 

-- neue ISO-konforme Syntax
SELECT Interpret, COUNT(*) as Anzahl
FROM dbo.CD 
GROUP BY ROLLUP (Interpret)

SELECT Interpret, Titel, COUNT(*) as Anzahl
FROM dbo.CD 
GROUP BY ROLLUP (Interpret, Titel)

SELECT Interpret, COUNT(*) as Anzahl, 
	MIN(Erscheinungsjahr) as Von, MAX(Erscheinungsjahr) as Bis
FROM dbo.CD 
GROUP BY ROLLUP (Interpret)

-- 7.2 Komplexe INSERTs, UPDATEs & DELETEs --

TRUNCATE TABLE CDMitTracks

INSERT INTO CDMitTracks
SELECT CD.*, CDT.TrackNr, CDT.Titel as TrackTitel
FROM CD LEFT JOIN CD_Track as CDT ON CD.ID = CDT.idCD

--

UPDATE VW_AktuelleCDs
SET TrackBewertung=Bewertung
WHERE TrackBewertung IS NULL

UPDATE dbo.CD_Track
SET CD_Track.Bewertung=CD.Bewertung
FROM dbo.CD 
	INNER JOIN dbo.CD_Track ON CD.ID=CD_Track.idCD
WHERE CD_Track.Bewertung IS NULL

UPDATE dbo.CD_Track
SET Bewertung=(SELECT Bewertung FROM dbo.CD WHERE CD.ID=CD_Track.idCD)
WHERE CD_Track.Bewertung IS NULL

--

SELECT * INTO dbo.CD2 FROM dbo.CD

DELETE FROM dbo.CD2
FROM dbo.CD2 LEFT JOIN dbo.CD_Track ON CD2.ID = CD_Track.idCD
WHERE CD_Track.idCD IS NULL

SELECT * FROM dbo.CD2

DROP TABLE dbo.CD2
SELECT * INTO dbo.CD2 FROM dbo.CD

DELETE dbo.CD2
FROM dbo.CD2 LEFT JOIN dbo.CD_Track ON CD2.ID = CD_Track.idCD
WHERE CD_Track.idCD IS NULL

DROP TABLE dbo.CD2
SELECT * INTO dbo.CD2 FROM dbo.CD

DELETE FROM dbo.CD2
WHERE ID NOT IN (SELECT DISTINCT idCD FROM dbo.CD_Track)

DROP TABLE dbo.CD2

-- 7.3 Daten abgleichen mit dem MERGE-Befehl --

SELECT Top 1 *
INTO dbo.BuchImport
FROM dbo.Buch

UPDATE dbo.BuchImport
SET Kategorie = 'Fachbuch'

INSERT INTO dbo.BuchImport (Autor, Titel, Kategorie)
VALUES ('Robert Panther', 'SQL-Abfragen optimieren', 'Fachbuch')

SELECT * FROM dbo.BuchImport

-- Alte Variante mit update, insert & delete:
UPDATE dbo.Buch
SET Buch.Autor = BuchImport.Autor,
	Buch.Titel = BuchImport.Titel,
	Buch.Kategorie = BuchImport.Kategorie
FROM dbo.Buch
	INNER JOIN dbo.BuchImport ON Buch.Titel=BuchImport.Titel

INSERT INTO dbo.Buch (Autor, Titel, Kategorie)
SELECT Autor, Titel, Kategorie
FROM dbo.BuchImport
WHERE Titel NOT IN (SELECT Titel FROM dbo.Buch)

DELETE FROM dbo.Buch
WHERE Titel NOT IN (SELECT Titel FROM dbo.BuchImport)

-- Neue Variante mit merge:
SELECT * FROM dbo.Buch
SELECT * FROM dbo.BuchImport

MERGE INTO dbo.Buch
USING BuchImport ON Buch.Titel=BuchImport.Titel
WHEN MATCHED
	THEN UPDATE SET Buch.Autor = BuchImport.Autor,
		Buch.Titel = BuchImport.Titel,
		Buch.Kategorie = BuchImport.Kategorie
WHEN NOT MATCHED
	THEN INSERT (Autor, Titel, Kategorie) 
	VALUES (BuchImport.Autor, BuchImport.Titel, BuchImport.Kategorie)
WHEN NOT MATCHED BY SOURCE THEN
	DELETE;

SELECT * FROM dbo.Buch

-- 7.4 Common Table Expressions --

WITH CTE AS
(
	SELECT idCD, COUNT(*) AS AnzahlSongs
	FROM dbo.CD_Track
	GROUP BY idCD
)
SELECT CD.Interpret, CD.Titel, CTE.AnzahlSongs
FROM dbo.CD as CD
	INNER JOIN CTE on CD.id=CTE.idCD

SELECT CD.Interpret, CD.Titel, COUNT(*) AS AnzahlSongs
FROM dbo.CD as CD
	INNER JOIN dbo.CD_Track AS CDT on CD.id=CDT.idCD
GROUP BY CD.Interpret, CD.Titel

-- 7.5 Die Offset-Klausel --
GO

WITH CTE1 AS
(
	SELECT TOP 20 *
	FROM dbo.CD_Track
	ORDER BY ID 
),
CTE2 AS
(
	SELECT TOP 10 *
	FROM CTE1 
	ORDER BY ID DESC
)
SELECT *
FROM CTE2
ORDER BY ID

SELECT *
FROM dbo.CD_Track
WHERE ID BETWEEN 11 AND 20

SELECT *
FROM dbo.CD_Track
ORDER BY ID 
	OFFSET 10 ROWS
	FETCH NEXT 10 ROWS ONLY

-- 7.6 Sequenzen --

CREATE SEQUENCE dbo.MediaId START WITH 1 INCREMENT BY 1

SELECT NEXT VALUE FOR dbo.MediaId

SELECT * FROM sys.sequences

ALTER TABLE dbo.Buch
ADD MediaId INT NULL DEFAULT (NEXT VALUE FOR dbo.MediaId)

ALTER TABLE dbo.CD 
ADD MediaId INT NULL DEFAULT (NEXT VALUE FOR dbo.MediaId)

ALTER TABLE dbo.DVD
ADD MediaId INT NULL DEFAULT (NEXT VALUE FOR dbo.MediaId)

INSERT INTO dbo.CD (Titel, Interpret)
VALUES ('Greatest Hits 2', 'Queen')

INSERT INTO dbo.Buch (Titel, Autor)
VALUES ('Datenbanken entwicklen mit SQL Server 2012', 'Robert Panther')

SELECT * FROM dbo.CD
SELECT * FROM dbo.Buch

UPDATE dbo.Buch SET MediaId = NEXT VALUE FOR dbo.MediaId WHERE MediaId IS NULL

SELECT * FROM dbo.Buch

UPDATE dbo.CD SET MediaId = NEXT VALUE FOR dbo.MediaId WHERE MediaId IS NULL
UPDATE dbo.DVD SET MediaId = NEXT VALUE FOR dbo.MediaId WHERE MediaId IS NULL

SELECT * FROM dbo.CD
SELECT * FROM dbo.DVD

-- 7.7 Temporale Tabellen --

USE MediaBase
GO

CREATE TABLE dbo.BuchTemp
(
  ID int IDENTITY(1,1) NOT NULL,
  ISBN10 char(10) NULL,
  ISBN13 char(14) NULL,
  Autor varchar(80) NULL,
  Titel varchar(80) NULL,
  Verlag varchar(80) NULL,
  Seiten int NULL,
  Auflage tinyint NULL,
  Sprache varchar(20) NULL,
  Hardcover bit NULL,
  Erscheinungsjahr int NULL,
  Bewertung tinyint NULL,
  Beschreibung varchar(max) NULL,
  Kategorie varchar(80) NULL,
  MediaId int NULL,

  SystemTimeStart datetime2 GENERATED ALWAYS AS ROW START NOT NULL,
  SystemTimeEnd datetime2 GENERATED ALWAYS AS ROW END NOT NULL,
  PERIOD FOR SYSTEM_TIME (SystemTimeStart, SystemTimeEnd),

  CONSTRAINT PK_BuchTemp PRIMARY KEY CLUSTERED (ID ASC)
)
WITH (SYSTEM_VERSIONING = ON (HISTORY_TABLE = dbo.BuchTemp_Hist));
--WITH (SYSTEM_VERSIONING = ON (HISTORY_TABLE = dbo.BuchTemp_Hist, HISTORY_RETENTION_PERIOD = 1 WEEK));
GO
-- ohne Angabe des Namens wird automatisch einer vergeben

ALTER TABLE dbo.BuchTemp ADD CONSTRAINT DF_BuchTemp_Sprache DEFAULT ('deutsch') FOR Sprache
GO

ALTER TABLE dbo.BuchTemp ADD DEFAULT (NEXT VALUE FOR dbo.MediaId) FOR MediaId
GO

-- History-Tabelle dbo.BuchTemp_Hist hat identische Struktur, 
-- aber ohne Constraints und mit anderen Indizes (zustzliche Indizes knnen manuell angelegt werden)

SET IDENTITY_INSERT dbo.BuchTemp ON

INSERT INTO dbo.BuchTemp (ID, ISBN10, ISBN13, Autor, Titel, Verlag, Seiten, Auflage, Sprache, Hardcover, Erscheinungsjahr, Bewertung, Beschreibung, Kategorie, MediaId)
SELECT ID, ISBN10, ISBN13, Autor, Titel, Verlag, Seiten, Auflage, Sprache, Hardcover, Erscheinungsjahr, Bewertung, Beschreibung, Kategorie, MediaId
FROM dbo.Buch

SET IDENTITY_INSERT dbo.BuchTemp OFF

UPDATE dbo.BuchTemp
SET Autor = 'Robert F. Panther'
WHERE ID = 1

SELECT * FROM dbo.BuchTemp
SELECT * FROM dbo.BuchTemp_Hist

SELECT * FROM dbo.BuchTemp
FOR SYSTEM_TIME AS OF '2018-06-25 T22:12:00'

SELECT * FROM dbo.BuchTemp
FOR SYSTEM_TIME BETWEEN '2018-06-25 T22:00:00' AND '2018-06-25 T22:15:00'

TRUNCATE TABLE dbo.BuchTemp
-- Fehler!

DELETE FROM dbo.BuchTemp

SELECT * FROM dbo.BuchTemp
SELECT * FROM dbo.BuchTemp_Hist

DROP Table dbo.BuchTemp
-- Fehler!

ALTER TABLE dbo.BuchTemp SET (SYSTEM_VERSIONING = OFF)
-- Hist-Tabelle erscheint als normale Tabelle

DROP Table dbo.BuchTemp
DROP Table dbo.BuchTemp_Hist

--

SELECT IS_TEMPORAL_HISTORY_RETENTION_ENABLED, NAME FROM sys.databases

ALTER TABLE dbo.BuchTemp SET (SYSTEM_VERSIONING = ON (History_Retention_Period = INFINITE))
ALTER TABLE dbo.BuchTemp SET (SYSTEM_VERSIONING = ON (History_Retention_Period = 1 DAY));
ALTER TABLE dbo.BuchTemp SET (SYSTEM_VERSIONING = ON (History_Retention_Period = 1 WEEK));
ALTER TABLE dbo.BuchTemp SET (SYSTEM_VERSIONING = ON (History_Retention_Period = 1 MONTH));
ALTER TABLE dbo.BuchTemp SET (SYSTEM_VERSIONING = ON (History_Retention_Period = 1 YEAR));

SELECT NAME, TEMPORAL_TYPE_DESC, HISTORY_RETENTION_PERIOD, HISTORY_RETENTION_PERIOD_UNIT 
FROM sys.tables
