-- Kapitel 18 --

-- 18.1 Knoten und Kanten anlegen

-- Knoten anlegen

CREATE SCHEMA Graph
GO

CREATE TABLE Graph.Person
(
	ID int IDENTITY(1,1) NOT NULL,
	Nachname varchar(80) NULL,
	Vorname varchar(80) NULL
	PRIMARY KEY CLUSTERED (ID ASC)
)
AS NODE ON [PRIMARY]

CREATE TABLE Graph.Film
(
	ID int IDENTITY(1,1) NOT NULL,
	Titel varchar(80) NULL,
	Originaltitel varchar(80) NULL,
	Jahr int,
	Kategorie varchar(80) NULL
	PRIMARY KEY CLUSTERED (ID ASC)
)
AS NODE ON [PRIMARY]

CREATE TABLE Graph.CD
(
	ID int IDENTITY(1,1) NOT NULL,
	Titel varchar(80) NULL,
	Jahr int,
	Kategorie varchar(80) NULL
	PRIMARY KEY CLUSTERED (ID ASC)
)
AS NODE ON [PRIMARY]

GO

-- Kanten anlegen

CREATE TABLE Graph.Mag 
AS EDGE ON [PRIMARY]
GO

CREATE TABLE Graph.IstBeteiligtAn
(
	Funktion varchar(80) NOT NULL
)
AS EDGE ON [PRIMARY]
GO

CREATE UNIQUE NONCLUSTERED INDEX IX_Mag ON Graph.Mag ($from_id, $to_id);
GO

CREATE UNIQUE NONCLUSTERED INDEX IX_IstBeteiligtAn ON Graph.IstBeteiligtAn ($from_id, $to_id);
GO

-- 18.2 Daten in eine Graphdatenbank einfgen

-- Knoten einfgen

INSERT INTO Graph.Person (Nachname, Vorname)
VALUES ('Panther', 'Robert'),
	('Duckless', 'Cork'),
	('Duckless', 'Maik'),
	('Cupper', 'Jerry'),
	('Lori', 'Huck');

INSERT INTO Graph.Film (Titel, Originaltitel, Jahr, Kategorie)
VALUES  
	('Dr. Hausboot', 'Dr. Houseboat', 2004, 'Drama'),
	('11:45 Mittags', 'Almost high noon', 1952, 'Western'),
	('Getting Up', 'Getting Up', 1993, 'Drama');

INSERT INTO Graph.CD (Titel, Jahr, Kategorie)
VALUES ('Let Me Sing', 2011, 'Jazz');

-- Kanten einfgen

INSERT INTO Graph.Mag($from_id, $to_id) 
VALUES (
	(SELECT $node_id FROM Graph.Person WHERE Nachname = 'Panther'), 
	(SELECT $node_id FROM Graph.Film WHERE Titel = '11:45 Mittags')
);

INSERT INTO Graph.Mag($from_id, $to_id) 
VALUES (
	(SELECT $node_id FROM Graph.Person WHERE Nachname = 'Panther'), 
	(SELECT $node_id FROM Graph.Person WHERE Nachname = 'Cupper')
);

INSERT INTO Graph.Mag($from_id, $to_id) 
VALUES (
	(SELECT $node_id FROM Graph.Person WHERE Nachname = 'Cupper'), 
	(SELECT $node_id FROM Graph.Person WHERE Nachname = 'Lori')
);

INSERT INTO Graph.IstBeteiligtAn($from_id, $to_id, Funktion) 
VALUES (
	(SELECT $node_id FROM Graph.Person WHERE Nachname = 'Cupper'), 
	(SELECT $node_id FROM Graph.Film WHERE Titel = '11:45 Mittags'),
	'Schauspieler'
);

INSERT INTO Graph.IstBeteiligtAn($from_id, $to_id, Funktion) 
VALUES (
	(SELECT $node_id FROM Graph.Person WHERE Nachname = 'Lori'), 
	(SELECT $node_id FROM Graph.Film WHERE Titel = 'Dr. Hausboot'),
	'Schauspieler'
);

INSERT INTO Graph.IstBeteiligtAn($from_id, $to_id, Funktion) 
VALUES (
	(SELECT $node_id FROM Graph.Person WHERE Nachname = 'Lori'), 
	(SELECT $node_id FROM Graph.CD WHERE Titel = 'Let Me Sing'),
	'Interpret'
);

INSERT INTO Graph.IstBeteiligtAn($from_id, $to_id, Funktion) 
VALUES (
	(SELECT $node_id FROM Graph.Person WHERE Vorname = 'Cork'), 
	(SELECT $node_id FROM Graph.Person WHERE Vorname = 'Maik'),
	'Vater'
);

INSERT INTO Graph.IstBeteiligtAn($from_id, $to_id, Funktion) 
VALUES (
	(SELECT $node_id FROM Graph.Person WHERE Vorname = 'Maik'), 
	(SELECT $node_id FROM Graph.Film WHERE Titel = 'Getting Up'),
	'Schauspieler'
);

-- 18.3 Daten einer Graphdatenbank abfragen

SELECT * FROM Graph.Person
SELECT * FROM Graph.Film
SELECT * FROM Graph.CD

SELECT * FROM Graph.Mag
SELECT * FROM Graph.IstBeteiligtAn

SELECT p.* , b.Titel
FROM Graph.Person p
	INNER JOIN dbo.Buch b ON b.Autor = p.Vorname + ' ' + p.Nachname

-- Wer mag welche Filme?
SELECT Person.Vorname, Person.Nachname, Film.Titel, Film.Kategorie
FROM Graph.Person, Graph.Mag, Graph.Film
WHERE MATCH (Person-(Mag)->Film)

-- Wer war an welchem Film beteiligt?
SELECT Person.Vorname, Person.Nachname, Funktion, Film.Titel, Film.Kategorie
FROM Graph.Person, Graph.IstBeteiligtAn, Graph.Film
WHERE MATCH (Person-(IstBeteiligtAn)->Film)

-- Wer war an welcher CD beteiligt?
SELECT Person.Vorname, Person.Nachname, Funktion, CD.Titel, CD.Kategorie
FROM Graph.Person, Graph.IstBeteiligtAn, Graph.CD
WHERE MATCH (Person-(IstBeteiligtAn)->CD)

-- Wer war an welcher Person beteiligt?
SELECT p1.Nachname, p1.Vorname, Funktion, p2.Nachname, p2.Vorname
FROM Graph.Person p1, Graph.IstBeteiligtAn, Graph.Person p2
WHERE MATCH (p1-(IstBeteiligtAn)->p2)

-- In welchem Film waren Personen beteiligt, deren Vater Cork Duckless ist?
SELECT p2.Nachname, p2.Vorname, b2.Funktion, Film.Titel, Film.Jahr
FROM Graph.Person p1, Graph.IstBeteiligtAn b1, Graph.Person p2, Graph.IstBeteiligtAn b2, Graph.Film
WHERE MATCH (p1-(b1)->p2-(b2)->Film)
	AND p1.Vorname = 'Cork' AND p1.Nachname = 'Duckless'

-- Wer mag wen?
SELECT p1.Nachname, p1.Vorname, p2.Nachname, p2.Vorname
FROM Graph.Person p1, Graph.Mag m, Graph.Person p2
WHERE MATCH (p1-(m)->p2)

SELECT p1.Nachname, p1.Vorname, p2.Nachname, p2.Vorname, p3.Nachname, p3.Vorname
FROM Graph.Person p1, Graph.Mag m1, Graph.Person p2, Graph.Mag m2, Graph.Person p3
WHERE MATCH (p1-(m1)->p2-(m2)->p3)

-- Was und wen mag jemand?
SELECT p1.Nachname, p1.Vorname
FROM Graph.Person p1, Graph.Mag m
WHERE MATCH (p1-(m))
-- Fehler !!

SELECT p.Nachname, p.Vorname, m.$to_id, 
	JSON_VALUE(m.[$to_id_200A8756C8EA444AA5F30627CB6554D3],'$.table') AS Tabelle,
	JSON_VALUE(m.[$to_id_200A8756C8EA444AA5F30627CB6554D3],'$.id') AS Id
FROM Graph.Person p
	INNER JOIN Graph.Mag m 
		ON m.[$from_id_C3ACB48510F94F4DBD4FDFCB56C0B4CC]
			=p.[$node_id_DA1C8B671A70437D81964F8593817610]

SELECT p.Nachname, p.Vorname, m.$to_id, 
	JSON_VALUE(m.$to_id,'$.table') AS Tabelle,
	JSON_VALUE(m.$to_id,'$.id') AS Id
FROM Graph.Person p
	INNER JOIN Graph.Mag m ON m.$from_id=p.$node_id

SELECT p.Nachname, p.Vorname, m.$to_id, 
	OBJECT_NAME(OBJECT_ID_FROM_NODE_ID(m.$to_id)) AS Tabelle,
	GRAPH_ID_FROM_NODE_ID(m.$to_id) AS Id
FROM Graph.Person p
	INNER JOIN Graph.Mag m ON m.$from_id=p.$node_id

-- 18.4 Daten einer Graphdatenbank ndern

SELECT * 
FROM Graph.Film
WHERE Titel = 'Getting Up'

UPDATE Graph.Film
SET Kategorie = 'Thriller'
WHERE Titel = 'Getting Up'

SELECT * 
FROM Graph.Film
WHERE Titel = 'Getting Up'

SELECT *
FROM Graph.IstBeteiligtAn

UPDATE Graph.IstBeteiligtAn
SET Funktion = 'Erzeuger'
WHERE Funktion = 'Vater'

UPDATE Graph.IstBeteiligtAn
SET Funktion = 'Erzeuger'
WHERE GRAPH_ID_FROM_EDGE_ID($edge_id) = 3

SELECT *
FROM Graph.IstBeteiligtAn

-- geht nicht bei berechneten technischen Spalten (also auch from und to), stattdessen Lschen und neu Einfgen erforderlich

-- Lschen von Knoten
SELECT *
FROM Graph.Person

DELETE 
FROM Graph.Person
WHERE ID=3

-- Lschen von Kanten
SELECT *
FROM Graph.IstBeteiligtAn

DELETE 
FROM Graph.IstBeteiligtAn
WHERE Funktion='Erzeuger'

DELETE 
FROM Graph.IstBeteiligtAn
WHERE GRAPH_ID_FROM_EDGE_ID($edge_id) = 6
