-- Kapitel 19 --

-- 19.1 Vorbereitung

-- Aktivieren von MLS / externen Skripts
EXEC sp_configure 'external scripts enabled', 1
RECONFIGURE WITH OVERRIDE

-- eventuell Dienstneustart erforderlich
-- auch Launchpad Dienst starten!

EXEC sp_configure 'external scripts enabled'

USE MediaBase
GO

-- 19.2 R

--- Beispiel 1: Hello World ---

EXEC sp_execute_external_script  @language =N'R', @script=N'print("Hello world!");'
GO

EXEC sp_execute_external_script  @language =N'R',
	@script=N'OutputDataSet <- InputDataSet;',
	@input_data_1 =N'SELECT ''Hello world!'' AS Meldung'
WITH RESULT SETS (([Meldung] NVARCHAR(20) NOT NULL));
GO

--- Beispiel 2: SQL-Abfrage ---

-- R-Skript
DECLARE @rscript NVARCHAR(MAX);
SET @rscript = N'OutputDataSet <- InputDataSet;'
 
-- SQL-Abfrage
DECLARE @sqlscript NVARCHAR(MAX);
SET @sqlscript = N'SELECT Id, Titel, Autor FROM dbo.Buch;';

EXEC sp_execute_external_script
	@language = N'R',
	@script = @rscript,
	@input_data_1 = @sqlscript
WITH RESULT SETS ((Id INT NOT NULL, Titel VARCHAR(80) NULL, Autor VARCHAR(80)));
GO

--- Beispiel 3: SQL-Abfrage mit Parameter ---

-- R-Skript
DECLARE @rscript NVARCHAR(MAX);
SET @rscript = N'OutputDataSet <- InputDataSet;'
 
-- SQL-Abfrage
DECLARE @sqlscript NVARCHAR(MAX);
DECLARE @autor VARCHAR(80);

SET @sqlscript = N'SELECT Id, Titel, Autor FROM dbo.Buch WHERE Autor = @autorfilter;';
SET @autor = 'Robert Panther';

EXEC sp_execute_external_script
	@language = N'R',
	@script = @rscript,
	@input_data_1 = @sqlscript,
	@params = N'@autorfilter varchar(80)',
	@autorfilter = @autor
WITH RESULT SETS ((Id INT NOT NULL, Titel VARCHAR(80) NULL, Autor VARCHAR(80)));

GO

--- Beispiel 4: SQL-Abfrage mit Parameter in USP gekapselt ---

CREATE PROCEDURE dbo.USP_TitelFromAutor_R (@autor VARCHAR(80))
AS
BEGIN

	-- R-Skript
	DECLARE @rscript NVARCHAR(MAX);
	SET @rscript = N'OutputDataSet <- InputDataSet;'
 
	-- SQL-Abfrage
	DECLARE @sqlscript NVARCHAR(MAX);

	SET @sqlscript = N'SELECT Id, Titel, Autor FROM dbo.Buch WHERE Autor = @autorfilter;';

	EXEC sp_execute_external_script
		@language = N'R',
		@script = @rscript,
		@input_data_1 = @sqlscript,
		@params = N'@autorfilter varchar(80)',
		@autorfilter = @autor
	WITH RESULT SETS ((Id INT NOT NULL, Titel VARCHAR(80) NULL, Autor VARCHAR(80)));
	
END
GO

EXEC dbo.USP_TitelFromAutor_R @autor = 'Robert Panther';
GO

-- 19.3 Python

--- Beispiel 1: Hello World ---

EXEC sp_execute_external_script  @language =N'Python', @script=N'print("Hello world!");'
GO

EXEC sp_execute_external_script  @language =N'Python',
	@script=N'OutputDataSet = InputDataSet;',
	@input_data_1 =N'SELECT ''Hello world!'' AS Meldung'
WITH RESULT SETS (([Meldung] NVARCHAR(20) NOT NULL));
GO

--- Beispiel 2: SQL-Abfrage ---
 
-- Python-Skript
DECLARE @pscript NVARCHAR(MAX);
SET @pscript = N'OutputDataSet = InputDataSet;'

-- SQL-Abfrage
DECLARE @sqlscript NVARCHAR(MAX);
SET @sqlscript = N'SELECT Id, Titel, Autor FROM dbo.Buch;';
 
EXEC sp_execute_external_script
	@language = N'Python',
	@script = @pscript,
	@input_data_1 = @sqlscript
WITH RESULT SETS ((Id INT NOT NULL, Titel VARCHAR(80) NULL, Autor VARCHAR(80)));
GO

--- Beispiel 3: SQL-Abfrage ---

-- Python-Skript
DECLARE @pscript NVARCHAR(MAX);
SET @pscript = N'
	df = InputDataSet
	OutputDataSet = df'

-- SQL-Abfrage
DECLARE @sqlscript NVARCHAR(MAX);
SET @sqlscript = N'
	SELECT cd.Interpret, cd.Titel, count(cdt.Id) AS TrackAnzahl
		FROM dbo.CD AS cd
		LEFT JOIN dbo.CD_Track AS cdt ON cd.Id=cdt.idCD
	GROUP BY cd.Interpret, cd.Titel;';
 
EXEC sp_execute_external_script
	@language = N'Python',
	@script = @pscript,
	@input_data_1 = @sqlscript
WITH RESULT SETS ((Interpret VARCHAR(80) NULL, Titel VARCHAR(80) NULL, TrackAnzahl INT));
GO

-- Fehler!! (IndentationError: unexpected indent)
 
-- Python-Skript
DECLARE @pscript NVARCHAR(MAX);
SET @pscript = N'
df = InputDataSet
OutputDataSet = df'

-- SQL-Abfrage
DECLARE @sqlscript NVARCHAR(MAX);
SET @sqlscript = N'
	SELECT cd.Interpret, cd.Titel, count(cdt.Id) AS TrackAnzahl
		FROM dbo.CD AS cd
		LEFT JOIN dbo.CD_Track AS cdt ON cd.Id=cdt.idCD
	GROUP BY cd.Interpret, cd.Titel;';
 
EXEC sp_execute_external_script
	@language = N'Python',
	@script = @pscript,
	@input_data_1 = @sqlscript
WITH RESULT SETS ((Interpret VARCHAR(80) NULL, Titel VARCHAR(80) NULL, TrackAnzahl INT));
GO

--- Beispiel 4: SQL-Abfrage (Gruppierung in Python) ---
 
-- Python-Skript
DECLARE @pscript NVARCHAR(MAX);
SET @pscript = N'
df = InputDataSet
df = df.groupby(["Interpret", "Titel"], as_index=False).count()
OutputDataSet = df'

-- SQL-Abfrage
DECLARE @sqlscript NVARCHAR(MAX);
SET @sqlscript = N'
SELECT cd.Interpret, cd.Titel, cdt.Titel AS TrackTitel
FROM dbo.CD AS cd
	LEFT JOIN dbo.CD_Track AS cdt ON cd.Id=cdt.idCD;';
 
EXEC sp_execute_external_script
	@language = N'Python',
	@script = @pscript,
	@input_data_1 = @sqlscript
WITH RESULT SETS ((Interpret VARCHAR(80) NULL, Titel VARCHAR(80) NULL, TrackAnzahl INT));
GO

--- Beispiel 5: SQL-Abfrage (mit zustzlicher Spalte) ---

-- Python-Skript
DECLARE @pscript NVARCHAR(MAX);
SET @pscript = N'
df = InputDataSet

# Daten nach Interpret und Titel gruppieren
df = df.groupby(["Interpret", "Titel"], as_index=False).count()

# neue Spalte hinzufgen
df["Kategorie"] = "Pop/Rock"
df["Kategorie"][df["Interpret"] == "Dave Brubeck"] = "Jazz"

OutputDataSet = df';

-- SQL-Abfrage
DECLARE @sqlscript NVARCHAR(MAX);
SET @sqlscript = N'
SELECT cd.Interpret, cd.Titel, cdt.Titel AS TrackTitel
FROM dbo.CD AS cd
	LEFT JOIN dbo.CD_Track AS cdt ON cd.Id=cdt.idCD;';
 
EXEC sp_execute_external_script
	@language = N'Python',
	@script = @pscript,
	@input_data_1 = @sqlscript
WITH RESULT SETS ((Interpret VARCHAR(80) NULL, Titel VARCHAR(80) NULL, TrackAnzahl INT, Kategorie VARCHAR(80)));
GO

--- Beispiel 6: SQL-Abfrage (mit Parameter) ---
 
-- Python-Skript
DECLARE @pscript NVARCHAR(MAX);
SET @pscript = N'
df = InputDataSet

# Daten nach Interpret und Titel gruppieren
df = df.groupby(["Interpret", "Titel"], as_index=False).count()

# neue Spalte hinzufgen
df["Kategorie"] = "Pop/Rock"
df["Kategorie"][df["Interpret"] == JazzBand] = "Jazz"

OutputDataSet = df';

-- SQL-Abfrage
DECLARE @sqlscript NVARCHAR(MAX);
SET @sqlscript = N'
SELECT cd.Interpret, cd.Titel, cdt.Titel AS TrackTitel
FROM dbo.CD AS cd
	LEFT JOIN dbo.CD_Track AS cdt ON cd.Id=cdt.idCD;';
 
EXEC sp_execute_external_script
	@language = N'Python',
	@script = @pscript,
	@input_data_1 = @sqlscript,
	@params = N'@JazzBand VARCHAR(80)',
	@JazzBand = 'Dave Brubeck'
WITH RESULT SETS ((Interpret VARCHAR(80) NULL, Titel VARCHAR(80) NULL, TrackAnzahl INT, Kategorie VARCHAR(80)));
GO
